; ===========================================================================
; M68K Sandbox v.1.0
; (C) 2010, Vladikcomper
; ---------------------------------------------------------------------------
; Console Engine
; ===========================================================================

; ===========================================================================
; Main initialization routine
; ===========================================================================

Console_Init:
		move.l	#0,(SysConsoleRow).w	; clear cursor position
		bsr.w	ClearWorkspace
		bra.w	RedrawWorkspace


; ===========================================================================
; Subroutine to scroll console down
; ===========================================================================

Console_Scroll:
		lea	(SysWorkspace+39).w,a5	; start from the second line

		move.l	#$2E4,d1		; 19 * 39 - 1
	@move:	move.b	(a5)+,-40(a5)		; move every byte one row up
		dbf	d1,@move

		lea	-39(a5),a5
		moveq	#38,d1			; 1  * 39 - 1
	@clear:	move.b	#0,(a5)+		; clear the last line
		dbf	d1,@clear
		rts
		

; ===========================================================================
; Subroutine to write a system message in console
;  d0 - message id
; ===========================================================================

Console_Log:
		lea	(ConsoleMsg).w,a0	; load messages array
		mulu.w	#20,d0
		lea	(a0,d0.w),a0		; load message we need to display
                                             
		move.w	(SysConsoleRow).w,d0
		move.w  (SysConsolePos).w,d1
		cmp.w	d1,d0			; is the cursor at the begining of the row?
		beq.s	@chkpos			; if yes, branch
		subq.w	#2,d1
		cmp.w	d1,d0			; is the cursor at the begining of the row?
		beq.s	@chkpos			; if yes, branch
		addi.w	#39,d0			; go to the next row otherwise

@chkpos:	cmpi.w	#$30C,d0		; is the row going out of screen?
		bne.s	@write			; if not, branch
		bsr.w   Console_Scroll
		subi.w	#39,d0

@write:
		lea	(SysWorkspace).w,a5	; load workspace
		move.w	d0,(SysConsoleRow).w	; set row pointer
		lea	(a5,d0.w),a5		; load Workspace destination (adda!!!)
		move.b	#1,(a5)+		; set second pallete line

		moveq	#19,d0
@loop:		move.b	(a0)+,(a5)+		; write message char by char
		dbf	d0,@loop

		addi.w	#39,(SysConsoleRow).w	; set pointer to the next row
		move.w	(SysConsoleRow).w,(SysConsolePos).w
		bra.w	RedrawWorkspace


; ===========================================================================
; Subroutine to write user-defined text in console
;  a0 - text ptr, text can take no more than 38 bytes
; ===========================================================================

Console_OutText:
		move.w	(SysConsoleRow).w,d0
		cmp.w	(SysConsolePos).w,d0	; is the cursor at the begining of the row?
		bne.s	@write			; if not, branch

		cmpi.w	#$30C,d0		; is the row going out of screen?
		bne.s	@shift			; if not, branch
		bsr.w   Console_Scroll		; scroll console output
		subi.w	#39,d0			; push row pointer back
		move.w	d0,(SysConsolePos).w	; push position
@shift:		addq.w	#2,(SysConsolePos).w	; add a space to shift text to the right a bit

@write:		lea	(SysWorkspace).w,a5	; load workspace
		adda.w  (SysConsolePos).w,a5	; load destination
		move.w	d0,d2			; move default row ptr to d2
		addi.w	#39,d2
		sub.w   (SysConsolePos).w,d2	; counter until next row

@loop:		move.b	(a0)+,d3		; test char
		beq.s	@quit			; if it's a end char, branch
		move.b	d3,(a5)+		; otherwise, write it
		subq.b	#1,d2			; was the next row reached?
		bne.s   @loop			; if not, branch
		addi.w	#39,d0			; move ptr to next row

		cmpi.w	#$30C,d0		; is the row going out of screen?
		bne.s	@chng			; if not, branch
		bsr.w   Console_Scroll		; scroll console output
		subi.w	#39,d0			; push row pointer back
		suba.w	#39,a5			; set pointer to previous row

	@chng:	move.b	#39-2,d2		; restore counter
		adda.w	#2,a5			; shift workspace pointer
		bra.s	@loop

@quit:          move.w	d0,(SysConsoleRow).w	; set row pointer
		move.l	a5,d0
		subi.l	#SysWorkspace,d0
		move.w	d0,(SysConsolePos).w	; set new position

		bra.w	RedrawWorkspace
		

; ===========================================================================
; Subroutine to output user-defined numbers
;  d5 - Output type (24 - long, 8 - word, 0 - byte)
;  d6 - Output mode (e.g. dec+signed)
; ===========================================================================

Console_OutNumber:
		move.l	(SysInputBuff).w,d3	; load number into d3
		lea	(SysOutputBuff).w,a5	; load output buffer

		tst.b	d6			; was the number marked as signed?
		bpl.s	Console_NumberMode	; if not, branch

		move.l	d3,d0			; copy number to d0
		lsr.l	d5,d0			; shift bits to the right depending on number type
		tst.b	d0			; we've got high byte, test it
		bpl.s	Console_NumberMode	; if it's positive, branch
		beq.s	Console_NumberMode	; if it's zero, branch
		move.b	#'-',(a5)+		; output the '-' char
		move.l	#$FFFFFFFF,d0
		move.b	d5,d4
		subi.b	#24,d4
		neg.b	d4
		lsr.l	d4,d0
		sub.l	d3,d0
		addi.l	#1,d0
		move.l	d0,d3

; ---------------------------------------------------------------------------

Console_NumberMode:
		btst	#1,d6		; is Dec bit set?
		beq.s	@testBIN	; if not, branch
		bra.s	Console_OutDEC

@testBIN:	move.b	#'$',(a5)+	; output the '$' char
		btst	#0,d6		; is BIN bit set?
		beq.s	Console_OutHEX	; if not, branch
		move.b	#'%',-1(a5)	; replace '$' with '%'
		addq.b	#7,d5		; add 7 to type to get bits count

; ---------------------------------------------------------------------------

Console_OutBIN:
		btst	#6,d6		; is Trim bit set?
		beq.s	@write		; if not, branch

	@trim:	btst	d5,d3		; is bit zero?
		bne.s	@write		; if not, branch
		subq.b	#1,d5
		bne.s	@trim

	@write:	move.b	#'0',(a5)+
		btst	d5,d3		; is the bit set?
		beq.s	*+8		; if not, branch
		move.b	#'1',-1(a5)
		dbf	d5,@write

		bra.w	Console_OutputBuffer

; ---------------------------------------------------------------------------

Console_OutDEC:
		lea	(DecArray).w,a1    
		move.l	#$FFFFFFFF,d0
		move.b	d5,d4
		subi.b	#24,d4
		neg.b	d4
		lsr.l	d4,d0
		and.l	d0,d3    
		moveq	#0,d4              
		moveq	#9,d5		; set counter

	@loop:	moveq	#0,d0
		move.l	(a1)+,d1

	@digit:	sub.l	d1,d3
		bcs.s	@trim
		addq.b	#1,d0
		moveq	#1,d4		; set trim-over flag
		bra.s	@digit

	@trim:	add.l	d1,d3
		tst.b	d4		; is trimming over?
		bne.s	@write		; if so, branch
		tst.b	d0		; is digit zero?
		beq.s	@next		; if yes, branch

	@write:	addi.b	#'0',d0
		move.b	d0,(a5)+
	@next:	dbf	d5,@loop

		bra.s	Console_OutputBuffer

; ---------------------------------------------------------------------------

Console_OutHEX:
		moveq	#1,d4		; set trim-over flag
		btst	#6,d6		; is Trim bit set?
		beq.s	@loop		; if not, branch
		moveq	#0,d4		; clear trim over flag

	@loop:	move.l	d3,d0		; copy num
		lsr.l	d5,d0		; shift to get current byte
		andi.w	#$FF,d0		; save only byte itself
		ror.w	#4,d0		; get hi 4 bits
		bsr.s	@write
		move.b	#0,d0		; clear hi bits
		rol.w	#4,d0		; get lo 4 bits
		bsr.s	@write
		bra.s	@next

	@write:	tst.b	d4		; is trim over flag set?
		bne.s	@cont		; if yes, branch
		tst.b	d0		; is digit 0?
		beq.s	@quit		; if yes, branch
		moveq	#1,d4
	@cont:	addi.b	#'0',d0
		cmpi.b	#$A+'0',d0	; is digit $A or more?
		bcs.s	*+6		; if not, branch
		addi.b	#'@'-'9',d0
		move.b	d0,(a5)+	; output digit
	@quit:	rts

	@next:	subq.b	#8,d5		; next byte
		bpl.s	@loop
		beq.s	@loop
		
		tst.b	d4		; is trim over flag set?
		bne.s   *+6		; if yes, branch
		move.b	#'0',(a5)+

; ---------------------------------------------------------------------------

Console_OutputBuffer:
		move.b	#0,(a5)+		; mark the end of string
		lea	(SysOutputBuff).w,a0	; load output buffer again, but into a0
		bra.w	Console_OutText		; and show buffer in the console!


; ===========================================================================
; Subroutine to break a row in the console
; ===========================================================================

Console_Break:
		cmpi.w	#$30C,(SysConsoleRow).w	; is the row going outta screen?
		bne.s	@inc			; if not, branch
		bsr.w   Console_Scroll
		bra.w	RedrawWorkspace

@inc:		addi.w	#39,(SysConsoleRow).w	; go to the next
		move.w	(SysConsoleRow).w,(SysConsolePos).w
		rts

