; ===========================================================================
; M68K Sandbox v.1.0
; (C) 2010, Vladikcomper
; ---------------------------------------------------------------------------
; Program's core
; ===========================================================================

ProgramStart:
		move	#$2700,sr	; disable interrupts
		moveq	#$40,d0
		move.b	d0,($A10009).l	; init port 1 (joypad 1)
		move.b	d0,($A1000B).l	; init port 2 (joypad 2)
		move.b	d0,($A1000D).l	; init port 3 (expansion)
		move.b	($A10001).l,d0
		andi.b	#$C0,d0
		move.b	d0,($FFFFFFF8).w ; get some MD data


; ---------------------------------------------------------------------------
; Code to load global pallete
; ---------------------------------------------------------------------------
; Note:	Only 2 colours on each row are used, which are text background
;	and foreground.
; ---------------------------------------------------------------------------

		lea	($C00000).l,a6
		lea	(FontPall).w,a0	; load font pallete
		cram	$2,d0		; start from the second colour
		moveq	#3,d5

LoadPal_Loop:
		move.l	d0,4(a6)	; set CRAM access
		move.l	(a0)+,(a6)	; fill first 2 colours on pallete row
		add.l	#$200000,d0	; next pallete row
		dbf	d5,LoadPal_Loop
		cram	0,4(a6)
		move.w	#$800,(a6)

; ---------------------------------------------------------------------------
; Code to load font graphics
; --------------------------------------------------------------------------- 
; Note:	Font graphics are presented in 1bpp format, so this code converts
;	them to 4bpp format before writing to VRAM.
; ---------------------------------------------------------------------------

		vram	0,4(a6)
		move.w	#$21*8-1,d5	; fill first $21 tiles
	@fill:	move.l	#$11111111,(a6)
		dbf	d5,@fill

		lea	(FontGfx).w,a0	; load font 1bpp gfx
		vram	$420,4(a6)
		move.w	#751,d5		; (94 tiles * 8 rows) - 1

LoadGfx_Loop:
		moveq	#0,d1
		moveq	#7,d4
		move.b	(a0)+,d0	; get bitfield representing 8 pixels
		beq.s	LoadGfx_Skip	; if it's empty, branch

	GfxConv_Loop:
		btst	d4,d0
		beq.s	@next		; if pixel is black, branch
		move.b	d4,d2
		subi.b	#7,d2
		neg.b	d2
		lsl.b	#2,d2		; multiple by 4 to get shift value
		moveq	#1,d3
		lsl.l	d2,d3		; get new pixel position
		add.l	d3,d1		; apply pixel
	@next:	dbf	d4,GfxConv_Loop

LoadGfx_Skip:	addi.l	#$11111111,d1	; increment all colours value
		move.l	d1,(a6)		; write 8 pixels to VRAM
		dbf	d5,LoadGfx_Loop
		
; ---------------------------------------------------------------------------
; We've done with all loadings, start main loop
; ---------------------------------------------------------------------------

		move.w	#0,(SysScreenId).w	; clear ScrId + ScrMode

MainLoop:
		moveq	#0,d0
		move.b	(SysScreenId).w,d0	; load screen mode
		lsl.w	#2,d0			; multiple by 4
		jsr	ScreenArray(pc,d0.w)
		bra.s	MainLoop

; ---------------------------------------------------------------------------
ScreenArray:
		bra.w	MainScreen		; [0]	program select
		bra.w	ConsoleScreen		; [1]	program output
		bra.w	RegistersScreen		; [2]	registers view
		bra.w	RAMViewScreen		; [3]	RAM preview
		bra.w	CodeViewScreen		; [4]	source code view
		rts

; ===========================================================================
; Subroutine to show error if there happened one
; ===========================================================================

ShowError:
		move	#$2700,sr
                movem.l	d0-a7,(SysRegBuff).w	; save registers
		moveq	#0,d0
		move.b	(SysErrorId).w,d0
		mulu.w	#20,d0
		lea	($C00000).l,a6
		lea	(ErrorArray).w,a0
		lea	(a0,d0.w),a0		; get error text
		vram	$C202,4(a6)		; set VRAM destination
		move.w	#1<<13,d1		; set second pallete line
		moveq	#19,d6

	@loop:	move.b  (a0)+,d1
		move.w	d1,(a6)
		dbf	d6,@loop
                		
	@wait:	bsr.w	ReadJoypads
		tst.b	(SysJoypadPress).w	; was any button pressed?
		beq.w	@wait			; if not, branch

                movem.l	(SysRegBuff).w,d0-a7	; restore registers
		move	#$2300,sr
		rte


; ===========================================================================
; Subroutine to redraw upper and bottom bars
; ===========================================================================

RedrawBars:
		move	#$2700,sr
		lea	($C00000).l,a6
		bsr.w	Clear_Bars
		moveq	#0,d0
		move.b	(SysStatusId).w,d0
		mulu.w	#38,d0
		lea	(StatusArray).w,a5
		lea	(a5,d0.w),a5		; load status text
		vram	$CD02,4(a6)		; set destination
		bsr.w	WriteStatus		; show text

		tst.b	(SysScreenId).w		; is screen id zero?
		beq.s	Redraw_ShowTitle 	; if yes, branch

; ---------------------------------------------------------------------------

		lea	(TabStr).w,a5		; load tab labels
                vram	$C082,4(a6)
                move.b	(SysScreenId).w,d0	; screen id will be currently selected tab
                subi.b	#4,d0			; 4 instead of 3, to make it one less
                neg.b	d0			; get cycle-realative value
                moveq	#3,d6

Redraw_TabsLoop:
		move.w	#3<<13,d1	; use fourth pallete line
		cmp.b	d0,d6		; is this tab selected?
		bne.s	@loop
		move.w	#2<<13,d1	; use second pallete line

	@loop:	move.b	(a5)+,d1
		bmi.s	@quit		; if tab label is over, branch
		move.w	d1,(a6)		; write char
		bra.s	@loop

	@quit:	move.w	#$6000,d1
		move.w	d1,(a6)		; fill map with #0 to have space between tabs
		move.w	d1,(a6)		;
		move.w	d1,(a6)		;
		dbf	d6,Redraw_TabsLoop
		rts

; ---------------------------------------------------------------------------

Redraw_ShowTitle:
                lea	(TitleStr).w,a5		; load title text
		vram	$C082,4(a6)		; set destination
		bsr.w	WriteStatus		; show text
		rts

; ---------------------------------------------------------------------------

WriteStatus:  
		moveq	#37,d6
		move.w	#$6000,d1
	@loop:	move.b	(a5)+,d1
		move.w	d1,(a6)
		dbf	d6,@loop
		rts

; ---------------------------------------------------------------------------

Clear_Bars:
                vram	$C000,d0	; first bar destination
		moveq	#2,d6
		bsr.s   Clear_Bars_Loop
                vram	$CC80,d0	; second bar destination
		moveq	#2,d6

Clear_Bars_Loop:
		move.l	d0,4(a6)
		moveq	#20,d5
	@fill:  move.l	#$60006000,(a6)
		dbf	d5,@fill
		addi.l	#$800000,d0		; go to the next row
		dbf	d6,Clear_Bars_Loop	; repeat for 3 rows
		rts


; ===========================================================================
; Subroutine to redraw program's workspace
; ===========================================================================

RedrawWorkspace:
		move	#$2700,sr
		lea	($C00000).l,a6
		lea	(SysWorkspace).w,a5	; load char array
		vram	$C202,d0
		moveq	#19,d6

Workspace_Loop:     
		moveq	#0,d1
		move.l	d0,4(a6)	; set VDP access
		move.b	(a5)+,d1	; number of pallete row
		swap	d1
		lsr.l	#3,d1		; set row pallete
		moveq	#37,d5

	@loop:	move.b	(a5)+,d1
		move.w	d1,(a6)
		dbf	d5,@loop

		addi.l	#$800000,d0	; go to the next row
		dbf	d6,Workspace_Loop
		rts


; ===========================================================================
; Subroutine to clear workspace
; ===========================================================================

ClearWorkspace:
		lea	(SysWorkspace).w,a5
		move.l	#$C2,d0			; (20 * 39) \ 4 - 1
	@loop:	move.l	#0,(a5)+
		dbf	d0,@loop
		rts


; ===========================================================================
; Main screen - program select
; ===========================================================================

MainScreen:
		move	#$2700,sr		; disable interrupts
		move.b	#0,(SysStatusId).w	; set status str id to #0
		bsr.w	RedrawBars
		lea	(SysTextState1).w,a0	; load text state
		move.l	#ProjectLst,(a0)	; load projects list
		move.b	#1,$F(a0)		; enable override
		bsr.w	TextView_Init

MainScreen_Loop:
		bsr.w	RenderFrame
		nop
		nop
		bsr.w	TextView_Process
		tpress	START			; is Start pressed?
		beq.s   MainScreen_Loop		; if not, branch
		move.b	#1,(SysScreenId).w	; go to Console screen
		rts

; ===========================================================================
; Subroutine to switch between modes
; ===========================================================================

ChangeMode:
		tpress	Start			; is Start pressed?
		beq.s	@chkLeft		; if not, branch
		move.w	#0,(SysScreenId).w
		rts

; ---------------------------------------------------------------------------

@chkLeft:	tpress	Left			; is Left pressed?
		beq.s	@chkRight		; if not, branch
		cmpi.b	#1,(SysScreenId).w	; is screen Console?
		bne.s	@normalLeft
		move.b	#5,(SysScreenId).w
@normalLeft:	subq.b	#1,(SysScreenId).w
		rts

; ---------------------------------------------------------------------------

@chkRight:	tpress	Right			; is Right pressed?
		beq.s	@return			; if not, branch
		cmpi.b	#4,(SysScreenId).w	; is screen Source Code?
		bne.s	@normalRight
		move.b	#0,(SysScreenId).w
@normalRight:	addq.b	#1,(SysScreenId).w
@return:	rts


; ===========================================================================
; Console screen - Program output
; ===========================================================================

ConsoleScreen:
		move	#$2700,sr		; disable interrupts

		tst.b	(SysModeId).w		; is mode 0?
		bne.s	ConsoleScr_Restore	; if not, branch

		move.b	#0,(SysRAMViewInit).w	; clear init flags
		move.b	#0,(SysTextState2+$E).w	;

		move.w	#$1FFF,d0		; $8000 \ 4 - 1
		lea	($FF0000),a0		; load RAM destination
		move.l	#0,(a0)+		; clear RAM
		dbf	d0,*-6
		
		moveq	#$C,d0			; 6 + 7 -1
		lea	(SysRegBuff).w,a0	; load program's registers buffer
		move.l	#0,(a0)+		; clear registers
		dbf	d0,*-6
		move.w	#$2700,(SysInputBuff+$34).w	; load sr state

		move.b	#1,(SysModeId).w	; set mode to #1 (program launch)
		bsr.w	Console_Init		; initialize the console
		moveq	#0,d0
		bsr.w	Console_Log		; log that we're going to start a program
		lea	(ProjectInc).w,a0	; load project inc array
		move.w	(SysTextState1+$A),d0	; load currently selected file
		lsl.w	#2,d0			; multiple by 4
		adda.w	d0,a0			; add to a0 to get item index
		move.l	(a0),(SysPragmaPC).w	; set start PC
		move.b	#1,(SysStatusId).w	; display "Busy" status
		move.b	#0,(SysConsoleInit).w
		bra.w	RedrawBars		; redraw bars, according to new mode

ConsoleScr_Restore:
		tst.b	(SysConsoleInit).w	; do we need to restore workspace?
		beq.s	ConsoleScr_MainLoop	; if not, branch

		move.w	#$C2,d0			; $30C \ 4 - 1
		lea	(SysWorkspace).w,a0
		move.l	$1000(a0),(a0)+		; restore workspace
		dbf	d0,*-4

		move.b	(SysConsoleInit).w,(SysStatusId).w
		bsr.w	RedrawBars
		bsr.w	RedrawWorkspace   
		move.b	#0,(SysConsoleInit).w

ConsoleScr_MainLoop:
		bsr.w	RenderFrame
		moveq	#0,d0
		move.b	(SysModeId).w,d0	; load system mode
		subq.b	#1,d0			; sub 1, because it can't be 0
		add.w	d0,d0			; double it
		move.w	ConsoleScr_Modes(pc,d0.w),d1
		jmp	ConsoleScr_Modes(pc,d1.w)

; ---------------------------------------------------------------------------

ConsoleScr_Modes:
@s		dc.w	ConsoleScr_Launch-@s	; [1]	Run or continue the program (run once)
		dc.w	ConsoleScr_ToPause-@s	; [2]	Prepare to pause a program (run once)
		dc.w	ConsoleScr_Pause-@s	; [3]	Program paused (loop)
		dc.w	ConsoleScr_Wait-@s	; [4]	Program delayed (loop)
		dc.w	ConsoleScr_ToFree-@s	; [5]	Prepare to terminate a program (run once)
		dc.w	ConsoleScr_Free-@s	; [6]	Program terminated (loop)

; ---------------------------------------------------------------------------

ConsoleScr_Launch:
		move.b	#5,(SysModeId).w	; use ToFree mode next time
		movem.l (SysRegBuff).w,d0-a6	; load program's register state
		move.w	(SysInputBuff+$34).w,sr	; load sr state
		jsr	*+6
		move.l	(SysPragmaPC).w,(sp)	; stack magic, haha!
		rts

; ---------------------------------------------------------------------------

ConsoleScr_ToPause:
		addq.b	#1,(SysModeId).w
		moveq	#1,d0
		bsr.w	Console_Log		; log that we're going to pause a program
		move.b	#3,(SysStatusId).w	; set status that displays controls
		bra.w	RedrawBars		; redraw bars, to apply new status


ConsoleScr_Pause:
		tpress	C
		beq.s	@free
		move.b	#1,(SysModeId).w	; switch to continue mode
		move.b	#1,(SysStatusId).w	; display "Busy" status
		bra.w	RedrawBars		; redraw bars, according to new mode

	@free:	bra.s	ConsoleScr_Free

; ---------------------------------------------------------------------------

ConsoleScr_Wait:
		subq.b	#1,(SysDelayCounter).w
		bne.s	@quit
                move.b	#1,(SysModeId).w	; switch to continue mode
	@quit:	rts

; ---------------------------------------------------------------------------

ConsoleScr_ToFree:
		addq.b	#1,(SysModeId).w
		moveq	#2,d0
		bsr.w	Console_Log		; log that we're going to terminate a program
		move.b	#2,(SysStatusId).w	; set status that displays controls
		bra.w	RedrawBars		; redraw bars, to apply new status

ConsoleScr_Free:
		bsr.w	ChangeMode
		cmpi.b	#1,(SysScreenId).w	; are we going to leave this screen?
		beq.s	@quit			; if not, branch

		move.w	#$C2,d0			; $30C \ 4 - 1
		lea	(SysWorkspace).w,a0
		move.l	(a0)+,$FFC(a0)
		dbf	d0,*-6
		move.b	(SysStatusId).w,(SysConsoleInit).w

	@quit:	rts


; ===========================================================================
; Register control screen
; ===========================================================================

RegistersScreen:
		move	#$2700,sr		; disable interrupts
		move.b	#2,(SysStatusId).w	; set status
		bsr.w	RedrawBars		; redraw bars, according to new mode
		bsr.w	ClearWorkspace

		lea	(SysRegBuff).w,a0	; load program registers
		lea	(SysWorkspace).w,a5	; load workspace
		adda.w	#2+39,a5		; set destination
		move.w	#'d0',d1		; set register text
		moveq	#7,d6			; do 7 registers
		bsr.s	ShowRegisters

		lea	(SysWorkspace).w,a5	; load workspace
		adda.w	#23+39,a5		; set destination
		move.w	#'a0',d1		; set register text
		moveq	#6,d6			; do 6 registers
		bsr.s	ShowRegisters

		bsr.w	RedrawWorkspace
		bra.s	RegScr_Loop

ShowRegisters:
		movea.l	a5,a4                                         
		rol.w	#8,d1
		move.b	d1,(a4)+		; send register name 
		rol.w	#8,d1
		move.b	d1,(a4)+		; send register num
		addq.w	#1,d1			; change text for next register

		move.b	#'=',1(a4)
		move.b	#'$',3(a4)
		adda.w	#4,a4
		move.l	(a0)+,d0		; load register itself
		moveq	#7,d5

	@loop:	rol.l	#4,d0
		move.b	d0,d2
		andi.b	#$F,d2
		addi.b	#'0',d2
		cmpi.b	#$A+'0',d2		; is digit $A or more?
		bcs.s	*+6			; if not, branch
		addi.b	#'@'-'9',d2
		move.b	d2,(a4)+
		dbf	d5,@loop

		adda.w	#39*2,a5		; go to the next row
		dbf	d6,ShowRegisters
		rts

; ---------------------------------------------------------------------------

RegScr_Loop:
		bsr.w	RenderFrame
		nop
		nop
		bsr.w	ChangeMode
		cmpi.b	#2,(SysScreenId).w
		beq.s	RegScr_Loop
		rts
		

; ===========================================================================
; RAM View Screen
; ===========================================================================

RAMViewScreen:
		move	#$2700,sr		; disable interrupts
		move.b	#4,(SysStatusId).w	; set status
		bsr.w	RedrawBars
		move.b	#1,(SysRAMOverride).w	; set override flag
		bsr.w	RAMView_Init		; initialize the engine

RAMView_Loop:
		bsr.w	RenderFrame
		nop
		nop
		bsr.w	RAMView_Process
		bsr.w	ChangeMode
		cmpi.b	#3,(SysScreenId).w
		beq.s	RAMView_Loop
		rts

; ===========================================================================
; Source Code Viewing Screen
; ===========================================================================

CodeViewScreen:
		move	#$2700,sr		; disable interrupts
		move.b	#2,(SysStatusId).w	; set status
		bsr.w	RedrawBars
		lea	(SysTextState2).w,a0	; load text state
		lea	(ProjectScrArr).w,a1	; load project scr pointers array
		moveq	#0,d0
		move.w	(SysTextState1+$A),d0	; load currently selected file num
		lsl.w	#2,d0			; multiple by 4
		adda.l	d0,a1
		move.l	(a1),(a0)		; load text
		bsr.w	TextView_Init

CodeView_Loop:
		bsr.w	RenderFrame
		nop
		nop
		bsr.w	TextView_Process
		bsr.w	ChangeMode
		cmpi.b	#4,(SysScreenId).w
		beq.s	CodeView_Loop
		rts
