; ===========================================================================
; M68K Sandbox v.1.0
; (C) 2010, Vladikcomper
; ---------------------------------------------------------------------------
; Engine for viewing RAM
; ===========================================================================

; ===========================================================================
; Main initialization routine
; ===========================================================================

RAMView_Init:
		tst.b	(SysRAMViewInit).w	; does engine need to reinit?
		bne.s	RAMView_InitOver	; if not, branch
		move.l	#0,(SysRAMStartRow).w	; clear positions
		move.b	#1,(SysRAMViewInit).w	; set init flag
		
RAMView_InitOver:
		bsr.w	ClearWorkspace
		bra.w   RAMView_Update
		

; ===========================================================================
; Engine processing routine
; ===========================================================================

RAMView_Process:
		tpress	C			; is C pressed?
		beq.s	@chkB			; if not, branch
		moveq	#8-3,d1
		bra.s	RAMView_LargeScroll

@chkB:		tpress	B			; is B pressed?
		beq.s	@down_chkpress		; if not, branch
		moveq	#12-3,d1
		bra.s	RAMView_LargeScroll

@down_chkpress:
		tpress	Down			; is Down pressed?
		beq.s   @down_chkheld		; if not, branch
		move.b	#30,(SysKeyCounter).w	; set key delay for the first call
		bra.s	RAMView_ScrollDown

@down_chkheld:
		theld	Down			; is Down held?
		beq.s   @up_chkpress		; if not, branch
		subq.b	#1,(SysKeyCounter).w
		bne.s   RAMView_Process_Rts
		move.b	#5,(SysKeyCounter).w	; reset key counter with lower value
		bra.s	RAMView_ScrollDown

@up_chkpress:
		tpress	Up			; is Up pressed?
		beq.s	@up_chkheld		; if not, branch
		move.b	#30,(SysKeyCounter).w	; set key delay for the first call
		bra.s   RAMView_ScrollUp

@up_chkheld:
		theld	Up			; is Up held?
		beq.s   RAMView_Process_Rts	; if not, branch
		subq.b	#1,(SysKeyCounter).w
		bne.s   RAMView_Process_Rts
		move.b	#5,(SysKeyCounter).w	; reset key counter with lower value
		bra.s	RAMView_ScrollUp

RAMView_Process_Rts:
		rts


; ---------------------------------------------------------------------------
; Subroutine to large scrolling
;  d1 - scroll pitch (in bits)
; ---------------------------------------------------------------------------

RAMView_LargeScroll:
		move.w	(SysRAMCurrRow).w,d0	; load current row into d0
		moveq	#1,d2
		lsr.w	d1,d0
		lsl.w	d1,d0			; this need to clear bits
		lsl.w	d1,d2			; get add-factor
		add.w	d2,d0			; add it to addr
		cmpi.w	#$FFF,d0		; is addr overflowed?
		bcs.s	@apply			; if not, branch
		move.w	#0,d0			; clear it!
	@apply:	move.w	d0,d1
		swap	d0
		move.w	d1,d0
		move.l	d0,(SysRAMStartRow).w	; reset positions
		bra.w	RAMView_Update

; ---------------------------------------------------------------------------
; Subroutine to perform scrolling down
; ---------------------------------------------------------------------------

RAMView_ScrollDown:
		cmpi.w	#$FFF,(SysRAMCurrRow).w
		bne.s	@normal_scroll		; if it's not last row, branch
		tst.b	(SysRAMOverride).w	; is override flag set?
		beq.s	@return			; if not, branch
		move.l	#0,(SysRAMStartRow).w	; clear positions
		bra.s	@update

@normal_scroll:
		addq.w	#1,(SysRAMCurrRow).w	; add 1 to current row
		move.w	(SysRAMCurrRow).w,d0
		sub.w	(SysRAMStartRow).w,d0	; current row minus start row
		cmpi.b	#20,d0			; is current row going out of screen?
		bcs.s	@update			; if not, branch
		addq.w	#1,(SysRAMStartRow).w	; add 1 to start row

@update:	
		bra.w	RAMView_Update

@return:
		rts

; ---------------------------------------------------------------------------
; Subroutine to perform scrolling up
; ---------------------------------------------------------------------------

RAMView_ScrollUp:
		tst.w	(SysRAMCurrRow).w
		bne.s	@normal_scroll		; if it's not the first row, branch
		tst.b	(SysRAMOverride).w	; is override flag set?
		beq.s	@return			; if not, branch
		move.l	#$FEC0FFF,(SysRAMStartRow).w
		bra.s	@update

@normal_scroll:
		subq.w	#1,(SysRAMCurrRow).w	; sub 1 from current row
		move.w	(SysRAMCurrRow).w,d0
		cmp.w	(SysRAMStartRow).w,d0	; is current row going out of screen?
		bcc.s	@update			; if not, branch
		subq.w	#1,(SysRAMStartRow).w	; sub 1 from current row

@update:
		bra.w	RAMView_Update

@return:
		rts


; ===========================================================================
; Subroutine to display data in the workspace and update it
; ===========================================================================

RAMView_Update:
		move.w	(SysRAMStartRow).w,d0	; load start row  
		move.w	(SysRAMCurrRow).w,d2	; load currently selected row
		sub.w	d0,d2
		subi.w	#19,d2
		neg.w	d2			; get cycle-realative value
		lea	($FF0000).l,a0		; load start addr
		lsl.w	#3,d0			; multiple by 8
		adda.w	d0,a0			; add d0 to get proper addr
		lea	(SysWorkspace).w,a1	; load workspace
		moveq	#19,d5

RAMView_DisplayLoop:
		movea.l	a1,a2			; secondary workspace ptr
		move.b	#0,(a2)			; set first pallete line
		cmp.b	d2,d5			; does the row need lightning?
		bne.s	@skip			; if not, branch
		move.b	#2,(a2)			; set third pallete line
	@skip:	adda.w	#3,a2			; skip 3 chars
		move.b	#'$',(a2)+
		move.b	#'F',(a2)+
		move.b	#'F',(a2)+
		move.w	d0,d1
		ror.w	#8,d1
		bsr.s	@DisplayHEX
		move.w	d0,d1
		bsr.s	@DisplayHEX
		adda.w	#3,a2

		moveq	#7,d4
	@loop:	move.b	(a0)+,d1		; load byte from RAM
		bsr.s	@DisplayHEX		; display it
		adda.w	#1,a2			; skip 2 chars
		dbf	d4,@loop
		bra.s	@next
		
@DisplayHEX:	andi.w	#$FF,d1
		lsl.w	#4,d1
		lsr.b	#4,d1
		addi.w	#'00',d1
		rol.w	#8,d1			; swap the bytes
		cmpi.b	#$A+'0',d1		; is digit $A or more?
		bcs.s	*+6			; if not, branch
		addi.b	#'@'-'9',d1
		move.b	d1,(a2)+		; output first digit
		ror.w	#8,d1			; swap again
		cmpi.b	#$A+'0',d1		; is digit $A or more?
		bcs.s	*+6			; if not, branch
		addi.b	#'@'-'9',d1
		move.b	d1,(a2)+		; output second digit
		rts

	@next:	addi.w	#8,d0
		lea	39(a1),a1		; switch to the next row
		dbf	d5,RAMView_DisplayLoop

		bra.w	RedrawWorkspace
