; ===========================================================================
; M68K Sandbox v.1.0
; (C) 2010, Vladikcomper
; ---------------------------------------------------------------------------
; Main initialization code
; ===========================================================================

ROM_Start:	dc.l $FFFE00, EntryPoint, ErrorBus, ErrorAddress
		dc.l ErrorIllegal, ErrorZeroDivide, ErrorChkInstr, ErrorTrapInstr
		dc.l ErrorPrivilege, ErrorTrace, ErrorLine1010, ErrorLine1111
		dc.l ErrorException, ErrorException, ErrorException, ErrorException
		dc.l ErrorException, ErrorException, ErrorException, ErrorException
		dc.l ErrorException, ErrorException, ErrorException, ErrorException
		dc.l ErrorException, Error, Error, Error
		dc.l HBlank, Error, VBlank, Error
		dc.l Error, Error, Error, Error
		dc.l Error, Error, Error, Error
		dc.l Error, Error, Error, Error
		dc.l Error, Error, Error, Error
		dc.l Error, Error, Error, Error
		dc.l Error, Error, Error, Error
		dc.l Error, Error, Error, Error
		dc.l Error, Error, Error, Error
Header:		dc.b 'SEGA MEGA DRIVE '
		dc.b '(C)VLAD 2010.OCT'
		dc.b 'M68K SANDBOX BY VLADIKCOMPER                    '
		dc.b 'M68K SANDBOX BY VLADIKCOMPER                    '
		dc.b 'GM 00000000-00'
Checksum:	dc.w $0000
		dc.b 'J               '
		dc.l ROM_Start, ROM_End
		dc.l $FF0000,$FFFFFF
		dc.b $20,$20,$20,$20
		dc.l $20202020, $20202020
Notes:		dc.b '                                                    '
		dc.b 'JUE             '

; ===========================================================================

EntryPoint:
		tst.l	($A10008).l
		bne.s	PortA_Ok
		tst.w	($A1000C).l

PortA_Ok:
		bne.w	SkipSetup
		move.b	($A10001).l,d0
		andi.b	#$F,d0
		beq.s	SkipSecurity
		move.l	#'SEGA',($A14000).l

SkipSecurity:
		move.w	($C00004).l,d0
		moveq	#0,d0
		movea.l	d0,a6
		move.l	a6,usp
		lea	(SetupValues),a0
		move.w	#23,d1

SetupLoop:
		move.w	(a0)+,($C00004).l
		dbf	d1,SetupLoop
		move.l	#$40000080,($C00004).l
		move.w	#0,($C00000).l
		lea	($FFFF0000).l,a0
		move.w	#$3FFF,d1

RAMLoop:
		move.l	#0,(a0)+
		dbf	d1,RAMLoop
		move.w	#$8174,($C00004).l
		move.w	#$8F02,($C00004).l
		move.l	#$C0000000,($C00004).l
		move.w	#$3F,d1

CRAMLoop:
		move.w	#0,($C00000).l
		dbf	d1,CRAMLoop
		move.l	#$40000010,($C00004).l
		move.w	#$13,d1

VDPLoop:
		move.l	#0,($C00000).l
		dbf	d1,VDPLoop
		movem.l	($FF0000).l,d0-a6
		lea	($FFFE00).l,a7

SkipSetup:
		bra.w	ProgramStart

; ===========================================================================
SetupValues:
		dc.w	$8004, $8134, $8230, $8338, $8407, $857C
		dc.w	$8600, $8700, $8800, $8900, $8A00, $8B00
		dc.w	$8C81, $8D3F, $8E00, $8F02, $9001, $9100
		dc.w	$9200, $93FF, $94FF, $9500, $9600, $9780

; =========================================================================== 

		include 'macros.asm'
		include	'vars.asm'

; ===========================================================================

Error:
		nop
		nop
		bra.s	Error

ErrorException	raise	$0
ErrorBus	raise	$1
ErrorAddress	raise	$2
ErrorIllegal	raise	$3
ErrorZeroDivide	raise	$4
ErrorChkInstr	raise	$5
ErrorTrapInstr	raise	$6
ErrorPrivilege	raise	$7
ErrorTrace	raise	$8
ErrorLine1010	raise	$9
ErrorLine1111	raise	$A

; ===========================================================================
HBlank:
		rte

; ===========================================================================
VBlank:
		movem.l	d0-a6,-(sp)
		move.b	#0,(SysVBlaProcess).w
		btst	#6,($FFFFFFF8).w
		beq.s	@skip
		move.w	#$700,d0
		dbf	d0,*
@skip:          bsr.s   ReadJoypads
		movem.l	(sp)+,d0-a6
		rte

; ===========================================================================
ReadJoypads:
                lea	(SysJoypadHeld).w,a0
                lea	($A10003).l,a1
		bsr.s	@read
		addq.w	#2,a1
	@read:	move.b	#0,(a1)
		nop
		nop
		move.b	(a1),d0
		lsl.b	#2,d0
		andi.b	#$C0,d0
		move.b	#$40,(a1)
		nop
		nop	
		move.b	(a1),d1
		andi.b	#$3F,d1
		or.b	d1,d0
		not.b	d0
		move.b	(a0),d1
		eor.b	d0,d1
		move.b	d0,(a0)+
		and.b	d0,d1
		move.b	d1,(a0)+
		rts

; ===========================================================================
RenderFrame:
		move.b	#1,(SysVBlaProcess).w
		move	#$2300,sr
	@wait:	tst.b	(SysVBlaProcess).w
		bne.s	@wait
		rts

; ===========================================================================

		include	'core.asm'	; program's core
		include	'console.asm'	; console engine
		include	'textview.asm'	; Text View engine
		include	'ramview.asm'	; RAM View engine     
		include 'data.asm'	; data inclusions

; ===========================================================================

		include	'data\prgcmd.asm'	; extra commands for programs
ProjectInc:	include 'data\prginc.asm'	; project inclusions
ProjectLst:	incbin	'data\prglst.dat'	; project list
ProjectScrArr:	include	'data\prgsrc.asm'	; project source code ptrs
ProjectScr:	incbin	'data\prgscr.dat'	; project parsed source code

; ===========================================================================

ROM_End:
		END
