; ===========================================================================
; M68K Sandbox v.1.0
; (C) 2010, Vladikcomper
; ---------------------------------------------------------------------------
; Engine for viewing the text
; ===========================================================================

; State Format:	$0 [l]	Pointer to the beginning of the text
;		$4 [l]	Pointer to the current position of the text
;		$8 [w]	Start row
;		$A [w]	Current row
;		$C [w]	The amount of rows (sets on init)
;		$E [b]	Init flag (set if state was already initialized)
;		$F [b]	Override flag (allows overriding)

; ===========================================================================
; Main initialization routine
;  a0	- text view state pointer
; ===========================================================================

TextView_Init:  
		tst.b	$E(a0)		; was the state already initialized?
		bne.s	TextView_InitOver ; if yes, branch
		moveq	#0,d0
		movea.l	(a0),a1		; load text chars array
		move.w	#0,$C(a0)	; clear amount of rows

TextView_CountLines:
		move.b	(a1)+,d0
		beq.s	TextView_InitSet	; = $0	- end
		bpl.s   TextView_CountLines	; < $80	- normal char
		addq.w	#1,$C(a0)		; increase rows counter
		bra.s	TextView_CountLines

TextView_InitSet:
		move.b	#1,$E(a0)	; set init flag
		move.l	(a0),4(a0)	; set current position to beginning
		move.l	#0,8(a0)	; clear row positions

TextView_InitOver:
		bsr.w	ClearWorkspace
		bra.w   TextView_Update

; ===========================================================================
; Engine processing routine
; ===========================================================================

TextView_Process:
		tpress	Down			; is Down pressed?
		beq.s   @down_chkheld		; if not, branch
		move.b	#30,(SysKeyCounter).w	; set key delay for the first call
		bra.s	TextView_ScrollDown

@down_chkheld:
		theld	Down			; is Down held?
		beq.s   @up_chkpress		; if not, branch
		subq.b	#1,(SysKeyCounter).w
		bne.s   TextView_Process_Rts
		move.b	#5,(SysKeyCounter).w	; reset key counter with lower value
		bra.s	TextView_ScrollDown

@up_chkpress:
		tpress	Up			; is Up pressed?
		beq.s	@up_chkheld		; if not, branch
		move.b	#30,(SysKeyCounter).w	; set key delay for the first call
		bra.s   TextView_ScrollUp

@up_chkheld:
		theld	Up			; is Up held?
		beq.s   TextView_Process_Rts	; if not, branch
		subq.b	#1,(SysKeyCounter).w
		bne.s   TextView_Process_Rts
		move.b	#5,(SysKeyCounter).w	; reset key counter with lower value
		bra.s	TextView_ScrollUp

TextView_Process_Rts:
		rts

; ---------------------------------------------------------------------------
; Subroutine to perform scrolling down
; ---------------------------------------------------------------------------

TextView_ScrollDown:
		move.w	$C(a0),d0
		cmp.w	$A(a0),d0	; compare current row and rows amount
		bne.s	@normal_scroll	; if it's not last row, branch
		tst.b	$F(a0)		; is override flag set?
		beq.s	@return		; if not, branch
		move.l	#0,8(a0)	; reset start positions
		move.l	(a0),4(a0)	; reset start text pointer
		bra.s	@update

@normal_scroll:
		addq.w	#1,$A(a0)	; add 1 to current row
		move.w	$A(a0),d0
		sub.w	8(a0),d0	; current row minus start row
		cmpi.b	#20,d0		; is current row going out of screen?
		bcs.s	@update		; if not, branch

		addq.w	#1,8(a0)	; add 1 to start row
		movea.l	4(a0),a1	; load current text pointer
	@loop:	move.b	(a1)+,d0	; find start of the next row
		bpl.s	@loop
		move.l	a1,4(a0)	; save new pointer
		
@update:
		bra.w   TextView_Update

@return:
		rts


; ---------------------------------------------------------------------------
; Subroutine to perform scrolling up
; ---------------------------------------------------------------------------

TextView_ScrollUp:
		tst.w	$A(a0)		; is it the first row?
		bne.s	@normal_scroll	; if not, branch
		tst.b	$F(a0)		; is override flag set?
		beq.s	@return		; if not, branch
		move.w	$C(a0),$A(a0)	; set row to the last
		move.w	$A(a0),d0
		subi.w	#19,d0
		bmi.s	@update		; if text takes one screen or less, branch
		beq.s	@update		;

		move.w	d0,8(a0)	; set start row
		movea.l	(a0),a1		; load start text pointer
		moveq	#0,d5
		move.w	d0,d5
		subq.w	#1,d5
	@loop:	move.b	(a1)+,d0	; find the start of $8(a0) row
		bpl.s	@loop
		dbf	d5,@loop
		move.l	a1,4(a0)	; save new pointer
		bra.s	@update

@normal_scroll:
		subq.w	#1,$A(a0)	; sub 1 from current row
		move.w	$A(a0),d0
		cmp.w	8(a0),d0	; is current row going out of screen?
		bcc.s	@update		; if not, branch
		
		subq.w	#1,8(a0)	; sub 1 from start row
		bne.s	@findrow	; if start row is not zero, branch
                move.l	(a0),4(a0)	; reset start text pointer
		bra.s	@update

@findrow:
		movea.l	4(a0),a1	; load current text pointer
		suba.l	#1,a1
	@loop2:	move.b	-(a1),d0	; find the start of previous row
		bpl.s	@loop2
		move.l	a1,4(a0)	; save new pointer
		addi.l	#1,4(a0)   

@update:
		bra.w   TextView_Update

@return:
		rts

; ===========================================================================
; Subroutine to display text in the workspace and update it
; ===========================================================================

TextView_Update:
		movea.l	4(a0),a1	; load text, from defined position
		lea	(SysWorkspace).w,a2 ; load workspace
		move.w	$A(a0),d0	; load currently selected row to d0
		sub.w	$8(a0),d0	; get onscreen row coords
		subi.w	#19,d0
		neg.w	d0		; get cycle-realative value
		moveq	#19,d6

TextView_DisplayLoop:
		move.b	#0,(a2)		; set first pallete line
		cmp.b	d0,d6		; does the row need lightning?
		bne.s	@skip		; if not, branch
		move.b	#2,(a2)		; set third pallete line
@skip:		lea	1(a2),a3	; workspace secondary pointer
		moveq	#37,d5

@rowloop:	move.b	(a1)+,(a3)+	; move chars from text to workspace
		bmi.s	@fillzero	; if $80 reached, fill the rest with zero
		beq.s   @fillzero	; if $0  reached, fill the rest + quit
		dbf	d5,@rowloop	; to avoid overflows, repeat no more than 38 times

@fixoverflow:	move.b	(a1)+,d1	; string overflowed, avoid breaking data
		bmi.s   @quit
		beq.s	@quitall
		bra.s	@fixoverflow

@fillzero:	move.b	#0,-(a3)	; clear last char
		move.b	#0,(a3)+	; fill the rest of row with zero bytes
                dbf	d5,@fillzero+4
                tst.b	-1(a1)		; was the text set to end?
		beq.s	@quitall	; if so, quit everything!

@quit:		lea	39(a2),a2	; go to the next workspace row
		dbf	d6,TextView_DisplayLoop

@quitall:	bra.w	RedrawWorkspace	; so yeah, we've done then, redraw!
